#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from pyramid.events import subscriber
from zope.component.interfaces import ISite
from zope.interface import implementer
from zope.lifecycleevent import IObjectAddedEvent
from zope.schema.fieldproperty import FieldProperty

from onf_website.component.hearing.interfaces import IHearingManagerTarget
from onf_website.component.location import ILocationManagerTarget
from onf_website.shared.common.security import SharedToolManagerRestrictionInfo
from onf_website.shared.resource import Resource
from onf_website.shared.resource.interfaces import IResourceInfo, IResourceManager, IResourceManagerFactory, \
    RESOURCE_CONTENT_TYPE
from pyams_content.component.paragraph import IParagraphFactorySettings
from pyams_content.component.theme import ICollectionsManagerTarget, IThemesManagerTarget
from pyams_content.reference.pictograms.interfaces import IPictogramManagerTarget
from pyams_content.shared.common.interfaces import IManagerRestrictionsFactory, ISharedContentFactory
from pyams_content.shared.common.types import TypedSharedTool
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import utility_config
from pyams_utils.traversing import get_parent


@implementer(IResourceManager, IParagraphFactorySettings, ILocationManagerTarget, IHearingManagerTarget,
             IPictogramManagerTarget, IThemesManagerTarget, ICollectionsManagerTarget)
class ResourceManager(TypedSharedTool):
    """Resource manager class"""

    shared_content_type = RESOURCE_CONTENT_TYPE

    shared_content_types_fields = IResourceInfo

    allowed_paragraphs = FieldProperty(IParagraphFactorySettings['allowed_paragraphs'])
    auto_created_paragraphs = FieldProperty(IParagraphFactorySettings['auto_created_paragraphs'])


@utility_config(provides=IResourceManagerFactory)
class ResourceManagerFactory(object):
    """Default resource manager factory"""

    def __new__(cls):
        return ResourceManager


@adapter_config(context=IResourceManager, provides=IManagerRestrictionsFactory)
def resource_manager_restrictions_factory(context):
    """Resource manager custom restrictions factory"""
    return SharedToolManagerRestrictionInfo


@adapter_config(context=IResourceManager, provides=ISharedContentFactory)
def resource_manager_content_factory(context):
    return Resource


@subscriber(IObjectAddedEvent, context_selector=IResourceManager)
def handle_added_resource_manager(event):
    """Register resource manager when added"""
    site = get_parent(event.newParent, ISite)
    registry = site.getSiteManager()
    if registry is not None:
        registry.registerUtility(event.object, IResourceManager)
